/* 
 * CSE142, 99 Winter
 * Homework 5
 * Valentine's Day
 *
 * Author:     Krystal Dalton
 * Student ID: 9721023
 * Section:    BF
 *	
 * STARTER PROGRAM
 *
 * To start with, your job is to figure out how this program works.
 *
 * Greeting Card Designer (Ultimate version)
 *
 *  The GP142 graphics window shows some buttons and the inside
 *  or outside (cover) of a greeting card.  The idea is that you 
 *  can design what you want the card to look like by adding and moving
 *  objects: 
 *    messages, message color and position
 *    shapes,   shape color and position
 *
 *   To place a message or a greeting, you click on an appropriate button, and 
 *   then you can click on the desired position in the card.  
 
 *	 In this starter version, you can do much less.
 *   You can change the card background, place one shape on the card,
 *   and change the shape back and forth between two possible kinds.
 */

#include "gp142.h"
#include <math.h>
#include <time.h>
	/* <time.h> is here only so that one function can be called
	to seed the random number generator.
	Put it here in your program and forget about it, 
	just in case you decide later you need to use random numbers.
	*/

#define TRUE    		1
#define FALSE   		0

/*
 * The state, or status you are in determines what happens when you click 
 * the card:
 * ADD SHAPE     -- you click the card, and a new object is placed there.
 *                  You get into this mode by clicking an ADD button.
 *                  After doing an ADD, you are put into SELECTED mode.
 * ADD MESSAGE   -- you click the card, and a new object is placed there. 
 *			        Process same as adding a shape.	
 * 
 * SELECTED      -- An object is selected. In this mode, the type 
 *                  of rotation applies to thecurrent object.
 * SELECT OBJECT -- User wants to select an object.  User clicks mouse near 
 *                  location of object
 */
 
   
/* Hint: you may need to add statuses when you make the program more complex.*/
#define STATUS_ADD_SHAPE 1 
#define STATUS_SELECTED 2
#define STATUS_ADD_MESSAGE 3
#define STATUS_SELECT_OBJECT 4

/* Configure the appearance of the buttons on the menu bar 
 * by setting the number of pixels of button height and width
 */
#define BUTTON_HEIGHT  	35
#define BUTTON_WIDTH 	100

/*change this if you add or delete buttons*/
#define NUM_BUTTONS 	6
#define SELECTED_COLOR  RED

/* Position of the card in the window */
#define PAGE_LLX (-GP142_XMAX + BUTTON_WIDTH +60)
#define PAGE_LLY (-GP142_YMAX + 100)
#define PAGE_WIDTH (GP142_XMAX - PAGE_LLX - 80)
#define PAGE_HEIGHT (GP142_YMAX - PAGE_LLY - 50)


/*Maximum distance between 2 points on the card*/
#define MAX_DIST (sqrt(pow(PAGE_HEIGHT, 2) + pow(PAGE_HEIGHT,2)));

/* For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.
 */
#define FILL 0
#define NOFILL 1

/* Types of objects we support.  (Perhaps not all types are used yet.)*/
#define OBJECT_NONE 0
#define OBJECT_SHAPE 1
#define OBJECT_MESSAGE 2

/*Maximum possible number of objects*/
#define POSSIBLE_OBJECTS 20
#define POSSIBLE_SHAPES 10
#define POSSIBLE_MESSAGES 10

/*if you add new types, you may need to change this*/
#define MAX_SHAPES 4
#define FIRST_SHAPE_NUM 0

/* These are the kinds of shapes we support.  You'll need to add more. */
#define SHAPE_HEART 0
#define SHAPE_STAR  1
#define SHAPE_FLOWER 2
#define SHAPE_COUPLE 3

/* Hint: once you understand how these shape types are used, you may decide
 * to do something similar with messages
 */
#define MAX_MESSAGES 5
#define FIRST_MESSAGE_NUM 4

#define MESSAGE_ILOVEU 4
#define MESSAGE_SUGAR_LIPS 5
#define MESSAGE_HEART 6
#define MESSAGE_HAPPY_VDAY 7
#define MESSAGE_SWEETHEART 8

/*Total number of object types supported by card*/
#define TOTAL_OBJECTS 9

/*Possible places for mouse clicks*/
#define CLICK_BUTTON 1  /* You clicked on SOME button (doesn't say which one)*/
#define CLICK_CARD 2    /* You clicked in the card area */
#define CLICK_OUTSIDE 3	/* You clicked outside the card area, not on a button*/	


/*******************  FUNCTION PROTOTYPES **************************/

/* Drawing functions */
void DrawBlankScreen (void);
void DrawButtons(int last_button);
void DrawCard(int *background_color, int *status, int *object_count, 
			  int *object_type, int object_num[], int object_color[], 
			  int object_x[], int object_y[]);

void DrawShape(int shape_type, int color, int bottom_x, int bottom_y);
void DrawHeart (int x, int y, int color);
void DrawStar (int x, int y, int color);
void DrawFlower (int x, int y, int color);
void DrawCouple (int x, int y, int color);

void WriteMessage (int message_type, int color, int bottom_x, int bottom_y);
void WriteIloveU (int x, int y, int color);
void WriteSugar_Lips (int x, int y, int color);
void WriteHeart (int x, int y, int color);
void WriteHappy_VDay (int x, int y, int color);
void WriteSweetheart (int x, int y, int color);


/* Mouse click handling functions */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int *shape_count,
					  int *message_count,int *current_object,
					  int *object_type, int object_num[], int object_color[], 
					  int object_x[], int object_y[]);
int ClassifyMouseClick(int mouse_x, int mouse_y);
void HandleButton(int x, int y, int *background_color, int object_color[],
				  int *status, int *object_count, int *current_object,
				  int *object_type, int object_num[]);

/* Add an object to the card. */
void AddShape(int x, int y, int *status, int *object_count, int *shape_count,
			   int *current_object, int *object_type, int object_num[], int object_color[], 
			   int object_x[], int object_y[]);
void AddMessage(int x, int y, int *status, int *object_count, int *message_count,
				 int *current_object, int *object_type, int object_num[], int object_color[],
				 int object_x[], int object_y[]);

/*Find closest object from mouse click*/
void closest_object(int x, int y, int *object_count, int *current_object,
					 int object_x[], int object_y[], int *status);
int distance (int x, int y);

/*To change the location of the object to location of mouse click*/
void change_position(int x, int y, int object_x[], int object_y[], int *current_object);

/*****************************************************************************

     MAIN

*****************************************************************************/

/* main() has two chief purposes: to contain the important variables and 
 *initialize them, and to contain the "event loop," which constantly looks 
 *to see if an event has happened, and then processes the event by calling 
 *an appropriate function. An example of an event is a mouse click.
 */
int main(void)
{
	int quit = FALSE;              /* becomes TRUE to exit program */
	int mouse_x, mouse_y;          /* coordinates of a mouse click */
	char key_hit;                  /* which key was hit            */
	
	
	/*variables and their default values*/
	int background_color = YELLOW;
	int status = STATUS_ADD_SHAPE;
	
	/* Each object will have a number telling what kind of object it is, a color,
	 * and an x and y coordinate.  You'll need to make these into arrays so you
	 * can support more than one object.
	 */
	int object_count = 0;           /* How many objects on the page? */
	int current_object;
	int message_count = 0;
	int shape_count = 0;
	int object_type = OBJECT_NONE;		/* Can also be a shape*/
	int object_num[POSSIBLE_OBJECTS];    /* If type is shape, either heart or star*/
	int object_color[POSSIBLE_OBJECTS];  /* Color of object */
	int object_x[POSSIBLE_OBJECTS];      /* Position of object*/
	int object_y[POSSIBLE_OBJECTS];
	
	int last_button = -1;           /* Keep track of the last button clicked.
	                                   This would be usefull when highlighting
	                                   the last button the user clicked */

	int event; /*the most recent GP142 event*/
	
	time_t t; /*needed only when initializing a random number generator*/
	

	/*
	 * INITIALIZATIONS
	 */
	srand (time(&t));	    /* Seed the random number generator. This is 
							   something you should do if you plan to use 
							   random numbers. Leave it here in your program 
							   and forget about it.*/
	

	/* These GP142 initializations don't need to be changed */
	GP142_open();           /* Open and init the GP142 Graphics Window      */
	GP142_show_text(0);     /* Hide the text window                         */
	GP142_logging(LOG_OFF); /* Change to LOG_ON if you want it.             */
	
	GP142_clear();	        /* This will clear the window & paint it white. */
	GP142_animate(ANI_RUN); /* Enable periodic events                       */
	
	
	
	/*
	 * The main event loop:
	 * --- ---- ----- -----
	 * wait for the next user action, such as a mouse click or keyboard
	 * keypress, or a periodic update event (allowing us to display the
	 * next frame of an animation -- not used directly in this program)
	 * and call the appropriate function to handle it.  Repeat this until 
	 * the user signals "quit".
	 */
	while (!quit) {  /*this is the famous "event loop" */
		
		/*start by redrawing everything */
		DrawBlankScreen ();
		DrawCard (&background_color, &status, &object_count, &object_type, 
				  object_num, object_color, object_x, object_y);
		DrawButtons(last_button);	  /* Show the buttons...*/ 
		
		/*Wait for something to happen */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_hit);
		
		/* Something just happened.  Figure out what it was, and react 
		   appropriately. */
		switch (event) {
		case GP142_QUIT:
			quit = TRUE;           /* set flag to terminate loop*/
			break;
			
		case GP142_KBD:  /*A key was pressed */
			/*Not needed for this program.*/ 
			break;
			
		case GP142_MOUSE:
			/* A mouse click has occurred. Call the mouse handler to 
			   decide where the click was and react as needed. */
			HandleMouseClick(mouse_x, mouse_y, &background_color, &status, 
					         &object_count, &shape_count, &message_count,
							 &current_object, &object_type, 
							 object_num, object_color, 
							 object_x, object_y);	
			
			break;
			
		case GP142_PERIODIC:
			/*An events has occurred simply because some time has passed,*/
			/*not currently used in the program*/		
			break;

		default:
			
			break;
			
		}  /*end switch */
		
	}  /* end event loop */
	
	
	/*
	 * WRAP-UP
	 */
	GP142_close();         /* clean up and close graphics window   */
	
	return 0;
}  /*end main */

/*****************************************************************************

     FUNCTIONS

*****************************************************************************/
/* 
 * For a blank screen just draw a white rectangle that covers the whole
 * GP142 window.
 */
void DrawBlankScreen (void) {
	int font_size = 14;

	GP142_rectangleXY (WHITE,
					-GP142_XMAX, -GP142_YMAX,
					GP142_XMAX, GP142_YMAX,
					FILL);

} /*end DrawBlankScreen */


/* 
 * Draws the buttons 
 */
void DrawButtons(int last_button)
{
/* To add buttons, you'll need to change the #defined constant NUM_BUTTONS
 * and add messages to this array.  Although we haven't covered 2-D arrays
 * yet, you should be able to play with this enough to get it going.
 * Look at course packet, page N-25, for a similar example.
 */
	char b_labels [NUM_BUTTONS][25] = {
		"Background color",
			"Object color",
			"Next, please",
			"Add Message",
			"Add Shape",
			"Select object",
	}; /*end labels*/
	
	int i, text_offset = -GP142_XMAX + 6, font_size = 12;
	int y, label_y;
	
	/* draw rectangles and labels for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		y = GP142_YMAX - i * BUTTON_HEIGHT;
		
		GP142_rectangleXY(BLACK,
			-GP142_XMAX + 4,
			y - 2,
			-GP142_XMAX + BUTTON_WIDTH,
			y - BUTTON_HEIGHT - 2, 1);
		
		
		label_y = y - BUTTON_HEIGHT/2;
		
		if (i == last_button){
			GP142_printfXY(SELECTED_COLOR, text_offset, label_y, 
			font_size, b_labels[i]);
		}
		else{
			GP142_printfXY(BLACK, text_offset, label_y, font_size, b_labels[i]);
		} 
	}	/*end for loop*/
}  /*end draw buttons*/


/*
 * Render (draw) the page and all its contents.
 */ 

void DrawCard(int *background_color, int *status, int *object_count, 
			  int *object_type, int object_num[], int object_color[], 
			  int object_x[], int object_y[]){
	
	int font_size = 16;
	int i;


	GP142_clear();
	/*
	 * Draw the background of the card.
	 */
	GP142_rectangleXY (*background_color,
		PAGE_LLX, PAGE_LLY,
		PAGE_LLX+ PAGE_WIDTH, PAGE_LLY+ PAGE_HEIGHT,
		FILL);
	/*
	 *Display Program Title
	 */
	GP142_printfXY(RED, PAGE_LLX, PAGE_LLY + PAGE_HEIGHT + 10, font_size + 10, 
		"Customized Valentine's Day Card");



	/*
	 *Draw current Shapes and Messages on card
	 */
	
	for (i=0; i <= *object_count; i++){
		if (object_num[i] >= FIRST_MESSAGE_NUM){
		WriteMessage (object_num[i], object_color[i], object_x[i], object_y[i]);
		}

	}
	for(i=0; i <= *object_count; i++){
		if (object_num[i] < FIRST_MESSAGE_NUM){
		DrawShape (object_num[i],object_color[i], object_x[i], object_y[i]);
		}
	}


	/*
	 * Write an appropriate status message below the background
	 */
	switch (*status) {
	case STATUS_ADD_SHAPE:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add a shape");
		break;
	case STATUS_SELECTED:
		GP142_printfXY (GREEN, PAGE_LLX, PAGE_LLY-40, font_size+3, "Object Selected. Click on the card to move it,");
		GP142_printfXY (GREEN, PAGE_LLX, PAGE_LLY-53, font_size+3, "or click 'color' or 'next' buttons");	
		break;
	case STATUS_ADD_MESSAGE:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add a message");
		break;
	case STATUS_SELECT_OBJECT:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"To select an object, click close to its origin");
		break;
	default:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Status error -- notify programmer");
		break;
	} /*end switch */
	
	
} /*end DrawPage */


/*
 * Draw one shape.
 */
void DrawShape(int shape_type, int color, int bottom_x, int bottom_y) {

	switch ((shape_type - FIRST_SHAPE_NUM) % MAX_SHAPES) {
	case SHAPE_HEART:
		DrawHeart (bottom_x, bottom_y, color);
		break;
	case SHAPE_STAR:
		DrawStar (bottom_x, bottom_y, color);
		break;
	case SHAPE_FLOWER:
		DrawFlower (bottom_x, bottom_y, color);
		break;
	case SHAPE_COUPLE:
		DrawCouple (bottom_x, bottom_y, color);
		break;
	default:
		break;

	} /*end switch*/

}


/*
 * Draw a heart on the card. A heart will be made by two overlapping
 * circles and a triangle.
 *
 * The pulsation is caused by adding a small, random amount to the 
 * heart height each time this function is called.
 */
#define HEART_WIDTH 35
void DrawHeart (int x, int y, int color) {

	int h = (HEART_WIDTH/2) + (rand () %4);

	GP142_circleXY (color, x - h + 5, y, h);
	GP142_circleXY (color, x + h - 5, y, h);
	GP142_triangleXY (color, x- h*2 +5, y-5,
						x + h*2 -5, y-5,
						x, y - h*2 -5, FILL);

}


/*
 * Draw a star on the card.
 *
 * We make the star twinkle by selecting the number of points randomly.  Each 
 * time it is redrawn, it is slightly different than the time before, which 
 * makes it appear to pulsate,
 */
#define STARPOINTS 5
#define STARRADIUS 40
#define PI 3.1416
void DrawStar (int x, int y, int color) {

	int sp;
	double angle;
	int x1, y1;

	sp = STARPOINTS + (rand () %3);
	for (angle = 0.0; angle < 2 * PI; angle = angle + (2*PI)/sp) {
		x1 = (int) (cos (angle) * STARRADIUS);
		y1 = (int) (sin (angle) * STARRADIUS);
		GP142_lineXY (color, x+x1, y+y1, x-x1, y-y1, 1);

	}
}

/*
 *Draws a flower on the card
 *
 *Six petals and one circle for the center of the flower are drawn.
 *Lines are used to represent the stem and leaves.
 */

void DrawFlower (int x, int y, int color) {
		int rad_pet=4;
		int rad_cntr=2;
		int stem_width=1;

		GP142_circleXY(color, x, y+8, rad_pet);
		GP142_circleXY(color, x, y-8, rad_pet);
		GP142_circleXY(color, x+8, y+4, rad_pet);
		GP142_circleXY(color, x+8, y-4, rad_pet);
		GP142_circleXY(color, x-8, y+4, rad_pet);
		GP142_circleXY(color, x-8, y-4, rad_pet);
		
		GP142_circleXY(color, x, y, rad_cntr);
		
		GP142_lineXY(color, x, y-12, x, y-28, stem_width);
		GP142_lineXY(color, x, y-28, x+6, y-22, stem_width);
		GP142_lineXY(color, x, y-28, x-6, y-22, stem_width);
}

/*
 *Draws a couple holding hands on the card
 *
 *Person is drawn as a stick figure, with a circle to represent the head 
 *and lines to represent the body
 */

void DrawCouple(int x, int y, int colors) {
	int rad_head=5;
	int body_width=7;
	int arm_leg_width=3;

	GP142_circleXY(colors, x+9, y+50, rad_head);	/*Draw Heads*/
	GP142_circleXY(colors, x-9, y+50, rad_head);

	GP142_lineXY(colors, x+9, y+40, x+9, y+20, body_width);	/*Draw Bodies*/
	GP142_lineXY(colors, x-9, y+40, x-9, y+20, body_width);

	GP142_lineXY(colors, x+9, y+38, x+18, y+20, arm_leg_width);	/*Draw Arms*/
	GP142_lineXY(colors, x+9, y+38, x, y+20, arm_leg_width);
	GP142_lineXY(colors, x-9, y+38, x-18, y+20, arm_leg_width);
	GP142_lineXY(colors, x-9, y+38, x, y+20, arm_leg_width);
	
	GP142_lineXY(colors, x+9, y+20, x+6, y, arm_leg_width);		/*Draw Legs*/
	GP142_lineXY(colors, x+9, y+20, x+12, y, arm_leg_width);
	GP142_lineXY(colors, x-9, y+20, x-6, y, arm_leg_width);
	GP142_lineXY(colors, x-9, y+20, x-12, y, arm_leg_width);
}

void WriteMessage (int message_type, int color, int bottom_x, int bottom_y){
	
	switch (((message_type - FIRST_MESSAGE_NUM) % MAX_MESSAGES) + FIRST_MESSAGE_NUM){
	
	case MESSAGE_ILOVEU:
		WriteIloveU(bottom_x, bottom_y, color);
		break;
	
	case MESSAGE_SUGAR_LIPS:
		WriteSugar_Lips(bottom_x, bottom_y, color);
		break;
	
	case MESSAGE_HEART:
		WriteHeart(bottom_x, bottom_y, color);
		break;
	
	case MESSAGE_HAPPY_VDAY:
		WriteHappy_VDay(bottom_x, bottom_y, color);
		break;
	
	case MESSAGE_SWEETHEART:
		WriteSweetheart(bottom_x, bottom_y, color);
		break;
	
	default:
		break;
	
	}/*end switch*/

}/*end WriteMessage*/


/*Writes "I love U"*/
void WriteIloveU (int x, int y, int color){
	int font_size = 20;
	int F = font_size + rand()%4;

	GP142_printfXY(color, x, y, F, "I Love U");
}


/*Writes "Sugar Lips"*/
void WriteSugar_Lips (int x, int y, int color){
	int font_size = 20;

	GP142_printfXY(color, x, y, font_size + rand()%5, "SUGAR");
	GP142_printfXY(color, x+90, y, font_size - rand()%5, "LIPS");
}


/*Writes "My Heart Beats For You"*/
void WriteHeart (int x, int y, int color){
	int font_size = 15;

	GP142_printfXY(color, x, y, font_size, "My Heart");
	GP142_printfXY(color, x+60, y, font_size + rand () %4, "Beats");
	GP142_printfXY(color, x+105, y, font_size, "For You");

}


/*Writes "Happy V-Day"*/
void WriteHappy_VDay (int x, int y, int color){
	int font_size = 25;

	GP142_printfXY(color, x, y + rand() %2, font_size, "HAPPY V-DAY");
}


/*Writes "Sweetheart"*/
void WriteSweetheart (int x, int y, int color){
	int font_size = 15;

	GP142_printfXY(color, x, y, font_size - rand() %5, "Sweetheart");

}


/*
 * Mouse clicks occur in one of three regions of interest: on buttons, 
 * on the card, or anywhere else.
 */
int  ClassifyMouseClick(int mouse_x, int mouse_y)
{
  if ((mouse_x < -GP142_XMAX + BUTTON_WIDTH) &&
      (mouse_y < GP142_YMAX) &&
      (mouse_y > (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT)))
    return CLICK_BUTTON;
  else if (mouse_x >= PAGE_LLX && mouse_x < PAGE_LLX + PAGE_WIDTH
	  && mouse_y >= PAGE_LLY && mouse_y < PAGE_LLY + PAGE_HEIGHT)
    return CLICK_CARD;
  else
    return CLICK_OUTSIDE;

}


/*
 * Decide where a mouse click occured, and react appropriately.
 */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int *shape_count,
					  int *message_count, int *current_object,
					  int *object_type, int object_num[], int object_color[], 
					  int object_x[], int object_y[])
{

	int mouse_click;
	mouse_click = ClassifyMouseClick(mouse_x, mouse_y);
			
	if (mouse_click == CLICK_BUTTON) {
				
		/* A button has been clicked.  Call a function to handle it*/
		/* most of the variables passed as args will NOT change*/
		HandleButton(mouse_x, mouse_y, background_color, object_color, status, 
					object_count, current_object, object_type, object_num);

	} else if (mouse_click == CLICK_CARD) {
	
		
		switch (*status){
		
		case STATUS_ADD_SHAPE:
			
			/* Add a shape, if there isn't one already */
			AddShape(mouse_x, mouse_y, status, object_count, shape_count, 
				current_object, object_type, object_num, object_color, 
				object_x, object_y);
			
			break;
		
		case STATUS_ADD_MESSAGE:
		
			/*Add a message, if there isn't one already*/
			AddMessage(mouse_x, mouse_y, status, object_count, message_count,
					current_object, object_type, object_num, object_color,
					object_x, object_y);

			break;

		case STATUS_SELECT_OBJECT:
			/*Select object closest to x and y coordinates of mouse click*/
			
			closest_object(mouse_x, mouse_y, object_count, current_object, 
							object_x, object_y, status);
			break;
		
		case STATUS_SELECTED:
			/*new mouse click is new x and new y location of object*/
			change_position(mouse_x, mouse_y, object_x, object_y, current_object);
			break;

		}/*end switch*/

	}/*end conditional*/

}/*end HandleMouseClick*/



/*Find closest object to location of mouse click*/
void closest_object(int x, int y, int *object_count, int *current_object, 
					 int object_x[], int object_y[], int *status){
	
	int i;
	double current_dist, nearest_dist; 
	
	nearest_dist = MAX_DIST;

	for(i=0; i<*object_count; i++){
		int dist_x = object_x[i] - x;
		int dist_y = object_y[i] - y;

		current_dist= distance (dist_x, dist_y);

		if(current_dist < nearest_dist){
			nearest_dist = current_dist;
			
			*current_object=i;

		}/*end conditional*/
	
	}/*end loop*/

	*status=STATUS_SELECTED;

}/*end closest_object*/


/*To find distance between two points on card*/
int distance (int x, int y){
	
	int dist;
	dist= (int)(sqrt(pow(x,2) + pow(y,2)));
	return (dist);

}/*end distance*/



/*To change location of object to where mouse was clicked on card*/
void change_position(int x, int y, int object_x[], int object_y[], int *current_object){
	
	
	object_x[*current_object]= x;
	object_y[*current_object]= y;

}/*end change_position*/

/* 
 * Figure out which button was clicked, if any, and do the right 
 * thing.
 */

void HandleButton(int x, int y, int *background_color, int object_color[], 
				  int *status, int *object_count, int *current_object,
				  int *object_type, int object_num[]){
	
	int buttonNum = 0;
	

	/* If we're outside of this area, we could not have clicked on
	* a button.
	*/
	if (x > -GP142_XMAX + BUTTON_WIDTH)  
		return ;	/*That's an error situation*/
	
					/* We know the mouse clicked in the column of buttons. Calculate
					 which one.*/
	buttonNum = (GP142_YMAX - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
	case 0:
		/* Change the background color */
		*background_color = (*background_color + 1) % MAX_COLORS;
		break;
	case 1:
		/*Change the object color*/
		object_color[*current_object]= (object_color[*current_object] + 1) % MAX_COLORS;
		break;
	case 2:
		/* Change the type of the currently selected object */
		if (*object_type == OBJECT_SHAPE){
			object_num[*current_object] = (object_num[*current_object] +1) % MAX_SHAPES;
		}
		
		if (*object_type == OBJECT_MESSAGE){
			if (object_num[*current_object] != 9){
			object_num[*current_object] = (object_num[*current_object] % TOTAL_OBJECTS) + 1;
			}
			else{
			object_num[*current_object] = FIRST_MESSAGE_NUM;
			}
		}
		break;
	case 3:
		/*Add another message to the card*/
		*status = STATUS_ADD_MESSAGE;
		*object_type = OBJECT_MESSAGE;
		break;

	case 4:
		/*Select object to change*/
		*status = STATUS_ADD_SHAPE;
		*object_type = OBJECT_SHAPE;
		break;
	case 5:
		/*Status changes to select an object*/
		*status=STATUS_SELECT_OBJECT;
		break;
	default:
		*status = 999;
		/*Bad news if this is ever reached! */
		break;
	}/*End switch*/
}/*End HandleButton*/
	





/* If there's room for another shape, add it to the card. */
void AddShape(int x, int y, int *status, int *object_count, int *shape_count,
			   int *current_object, int *object_type, int object_num[], int object_color[], 
			   int object_x[], int object_y[])
{
		int temp;

		if(*shape_count < POSSIBLE_SHAPES){		/*Add another shape, if max shapes have
												not been reached*/
		
		*object_count=*object_count +1;					
		*shape_count= *shape_count +1;

		temp = *object_count;
		
		*current_object = temp-1;		/*Array subscript (current object) gets a value of one less that
												the total count of objects*/

		*status= STATUS_SELECTED;
		*object_type = OBJECT_SHAPE;
		object_num[*current_object] = SHAPE_HEART;
		object_color[*current_object] = BLACK;
		object_x[*current_object] = x;
		object_y[*current_object] = y;
	
	}/*end conditional*/

}/*end AddShape*/

void AddMessage (int x, int y, int *status, int *object_count, int *message_count,
				 int *current_object, int *object_type, int object_num[], 
				 int object_color[], int object_x[], int object_y[])
{
	
	int temp;

	if (*message_count<POSSIBLE_MESSAGES){    /*Add another message if max messages
												have not been reached*/
	

		*object_count = *object_count + 1;					  
		*message_count = *message_count + 1;

		temp = *object_count;
			
		*current_object = temp - 1;
		
		*status = STATUS_SELECTED;
		*object_type = OBJECT_MESSAGE;
		object_num[*current_object] = MESSAGE_ILOVEU;
		object_color[*current_object] = BLACK;
		object_x[*current_object] = x;
		object_y[*current_object] = y;
	
	}/*end contional*/

}/*end AddMessage*/